@@ -1,22 +1,36 @@
+"""
+To install the required dependencies:
+
+  uv pip install "datachain[examples]"
+
+
+Example demonstrates using DataChain Pytorch data loader to stream image
+and text object from cloud storage. We pass extracted data to Open CLIP
+to compute similarity score between image and its caption.
+"""
+
 import open_clip
 import torch
 from torch.nn.functional import cosine_similarity
 from torch.utils.data import DataLoader
 
 import datachain as dc
+from datachain import C, func
 
 source = "gs://datachain-demo/50k-laion-files/000000/00000000*"
 
 
 def create_dataset():
-    imgs = dc.read_storage(source, type="image").filter(dc.C("file.path").glob("*.jpg"))
-    captions = dc.read_storage(source, type="text").filter(
-        dc.C("file.path").glob("*.txt")
+    imgs = dc.read_storage(source, type="image", anon=True).filter(
+        C("file.path").glob("*.jpg")
+    )
+    captions = dc.read_storage(source, type="text", anon=True).filter(
+        C("file.path").glob("*.txt")
     )
     return imgs.merge(
         captions,
-        on=dc.func.path.file_stem(imgs.c("file.path")),
-        right_on=dc.func.path.file_stem(captions.c("file.path")),
+        on=func.path.file_stem(imgs.c("file.path")),
+        right_on=func.path.file_stem(captions.c("file.path")),
     )
 
 
@@ -34,9 +48,9 @@ if __name__ == "__main__":
     )
     loader = DataLoader(ds, batch_size=16)
 
-    similarity_sum = 0
+    similarity_sum = 0.0
     row_count = 0
-    with torch.no_grad(), torch.cuda.amp.autocast():
+    with torch.no_grad(), torch.amp.autocast("cuda"):
         for image, text in loader:
             image_features = model.encode_image(image)
             text_features = model.encode_text(text)
